local NUM_TRADING_SLOTS = 7
local interfaceItem = {}
local interfaceIndex = 0

local TradeFill = LibStub("AceAddon-3.0"):NewAddon("TradeFill", "AceConsole-3.0", "AceEvent-3.0")

local classes = { "WARRIOR", "PALADIN", "HUNTER", "ROGUE", "PRIEST", "SHAMAN", "MAGE", "WARLOCK", "DRUID" }

local OptionFrame = CreateFrame("Frame", "MyIconButtonFrame", TradeFrame)
local ButtonA = CreateFrame("Button", "MyTradeButton", OptionFrame)
local ButtonB = CreateFrame("Button", "MyTradeButton", OptionFrame)
local ClearTrade = CreateFrame("Button", "MyTradeButton", OptionFrame, "UIPanelButtonTemplate")

local options = {
    name = "TradeFill",
    handler = TradeFill,
    type = "group",
    childGroups = "tab",
    args = {
        menu = {
            type = "group",
            name = "",
            order = 0,
            args = {
                toggleBoxA = {
                    type = 'toggle',
                    name = "Show only full stacks",
                    desc = "Hide stacks that are not fill in dropdown menu",
                    get = function(info) return TradeFill.itemdb.profile.show end,
                    set = function(info, value) TradeFill.itemdb.profile.show = value end,
                    order = 0,
                },
                separatorA = {
                    type = "description",
                    name = " ",
                    order = 1,
                },
                iconA = {
                    type = "description",
                    fontSize = "medium",
                    name = function(info) return TradeFill:GetName("A") end,
                    image = function(info) return TradeFill:SetIconImage("A") end,
                    imageWidth = 32,
                    imageHeight = 32,
                    order = 2,
                },
                itemA = {
                    type = "select",
                    width = 1.5,
                    name = "Item to trade",
                    desc = "Select a item",
                    values = function() return TradeFill:SetOptionItem("A") end,
                    get = function() return TradeFill:GetInputItem("A") end,
                    set = function(info, value) TradeFill:SetSelectItem("A", tonumber(value)) end,
                    order = 3,
                },
                separatorB = {
                    type = "description",
                    name = " ",
                    order = 4,
                },
                iconB = {
                    type = "description",
                    fontSize = "medium",
                    name = function(info) return TradeFill:GetName("B") end,
                    image = function(info) return TradeFill:SetIconImage("B") end,
                    imageWidth = 32,
                    imageHeight = 32,
                    order = 5,
                },
                itemB = {
                    type = "select",
                    width = 1.5,
                    name = "Item to trade",
                    desc = "Select a item",
                    values = function() return TradeFill:SetOptionItem("B") end,
                    get = function() return TradeFill:GetInputItem("B") end,
                    set = function(info, value) TradeFill:SetSelectItem("B", tonumber(value)) end,
                    order = 6,
                },
                separatorC = {
                    type = "description",
                    name = " ",
                    order = 7,
                },
                toggleBoxB = {
                    type = 'toggle',
                    name = "Only full stacks",
                    desc = "Fill the trade window only with full stacks",
                    get = function(info) return TradeFill.itemdb.profile.stack end,
                    set = function(info, value) TradeFill.itemdb.profile.stack = value end,
                    order = 8,
                },
                toggleBoxC = {
                    type = 'toggle',
                    name = "Hide buttons",
                    desc = "Don't show extra buttons in trade window",
                    get = function(info) return TradeFill.itemdb.profile.button end,
                    set = function(info, value) TradeFill.itemdb.profile.button = value end,
                    order = 9,
                },
                separatorD = {
                    type = "description",
                    name = " ",
                    order = 10,
                },
            },
            inline = true,
        },
        stacks = {
            type = "group",
            name = "Stacks",
            desc = "The number of stacks you want to trade for each class",
            order = 1,
            args = {
            },
        },
        group = {
            type = "group",
            name = "Condition",
            desc = "When to AutoFill the trade window",
            order = 2,
            args = {
                toggleBoxA = {
                    type = 'toggle',
                    name = "Strangers",
                    desc = "AutoFill trade window with anyone",
                    get = function() return TradeFill.itemdb.profile.solo end,
                    set = function(info, value) TradeFill.itemdb.profile.solo = value end,
                    order = 0,
                },
                toggleBoxB = {
                    type = 'toggle',
                    name = "Party members",
                    desc = "AutoFill trade window with party members",
                    get = function() return TradeFill.itemdb.profile.party end,
                    set = function(info, value) TradeFill.itemdb.profile.party = value end,
                    order = 1,
                },
                toggleBoxC = {
                    type = 'toggle',
                    name = "Raid members",
                    desc = "AutoFill trade window with raid members",
                    get = function() return TradeFill.itemdb.profile.raid end,
                    set = function(info, value) TradeFill.itemdb.profile.raid = value end,
                    order = 2,
                },
                group = {
                    type = "group",
                    name = "Optional",
                    desc = "Player also has to meet conditions",
                    order = 3,
                    args = {
                        toggleBoxD = {
                            type = 'toggle',
                            name = "Guild members",
                            desc = "Only AutoFill trade window with guild members",
                            get = function() return TradeFill.itemdb.profile.guild end,
                            set = function(info, value) TradeFill.itemdb.profile.guild = value end,
                            order = 5,
                        },
                        toggleBoxE = {
                            type = 'toggle',
                            name = "Required level",
                            desc = "Only AutoFill trade window when the player has minimum required item level",
                            get = function() return TradeFill.itemdb.profile.level end,
                            set = function(info, value) TradeFill.itemdb.profile.level = value end,
                            order = 6,
                        },
                    },
                },
            },
        },
        exclude = {
            type = "group",
            name = "Exclude",
            desc = "When not to fill the trade window",
            order = 3,
            args = {
                player = {
                    type = "group",
                    name = "Players",
                    order = 0,
                    args = {
                        inputA = {
                            type = 'input',
                            name = "Player Names",
                            desc = "Don't fill trade window with those players",
                            get = function() return TradeFill.itemdb.profile.players end,
                            set = function(info, value) TradeFill.itemdb.profile.players = value:gsub("\n*$", "") end,
                            pattern = "^[A-Za-z\n]*$",
                            usage = "Player names can only contains letters",
                            multiline = 10,
                            order = 0,
                        },
                    },
                },
                guild = {
                    type = "group",
                    name = "Guilds",
                    order = 1,
                    args = {
                        inputA = {
                            type = 'input',
                            name = "Guild Names",
                            desc = "Don't fill trade window with players from these guilds",
                            get = function() return TradeFill.itemdb.profile.guilds end,
                            set = function(info, value) TradeFill.itemdb.profile.guilds = value:gsub("\n*$", "") end,
                            pattern = "^[A-Za-z\n ]*$",
                            usage = "Guild names can only contains letters",
                            multiline = 10,
                            order = 1,
                        },
                    },
                },
            },
        },
    },
}

for _, class in pairs(classes) do
    local _, _, _, argbHex = GetClassColor(class)
    options.args.stacks.args[class] = {
        type = "group",
        name = "|c".. argbHex .. capitalizeFirstLetter(class) .. "|r",
        args = {
            iconA = {
                type = "description",
                fontSize = "medium",
                name = function(info) return TradeFill:GetName("A") end,
                image = function(info) return TradeFill:SetIconImage("A") end,
                imageWidth = 16,
                imageHeight = 16,
                order = 0,
            },
            selectedOptionA = {
                type = "select",
                width = 0.75,
                name = "Number of stacks",
                desc = "Choose an amount of stack you want to trade",
                values = function() return TradeFill:SetOptionStack("A") end,
                get = function() return tostring(TradeFill.stacksdb.profile[class].stackNumberA) end,
                set = function(info, value) TradeFill.stacksdb.profile[class].stackNumberA = tonumber(value) TradeFill:UpdateOptionStack(class, "A", tonumber(value)) end,
                order = 1,
            },
            separatorA = {
                type = "description",
                name = " ",
                order = 2,
            },
            iconB = {
                type = "description",
                fontSize = "medium",
                name = function(info) return TradeFill:GetName("B") end,
                image = function(info) return TradeFill:SetIconImage("B") end,
                imageWidth = 16,
                imageHeight = 16,
                order = 3,
            },
            selectedOptionB = {
                type = "select",
                width = 0.75,
                name = "Number of stacks",
                desc = "Choose an amount of stack you want to trade",
                values = function() return TradeFill:SetOptionStack("B") end,
                get = function() return tostring(TradeFill.stacksdb.profile[class].stackNumberB) end,
                set = function(info, value) TradeFill.stacksdb.profile[class].stackNumberB = tonumber(value) TradeFill:UpdateOptionStack(class, "B", tonumber(value)) end,
                order = 4,
            },
        },
    }
end

function TradeFill:enableDisableClassSetting(value, option)

    if option == "A" then

        if value == 0 then
            for _, class in pairs(classes) do
                options.args.stacks.args[class].args.selectedOptionA.disabled = true
            end
        else
            for _, class in pairs(classes) do
                options.args.stacks.args[class].args.selectedOptionA.disabled = false
            end
        end
    end

    if option == "B" then

        if value == 0 then
            for _, class in pairs(classes) do
                options.args.stacks.args[class].args.selectedOptionB.disabled = true
            end
        else
            for _, class in pairs(classes) do
                options.args.stacks.args[class].args.selectedOptionB.disabled = false
            end
        end
    end
end

function TradeFill:enableDisableButton(value, option)

    if option == "A" then

        if value == 0 or TradeFill.itemdb.profile.button == true then
            ButtonA:Hide()
        else
            ButtonA:Show()
        end
    end

    if option == "B" then

        if value == 0 or TradeFill.itemdb.profile.button == true then
            ButtonB:Hide()
        else
            ButtonB:Show()
        end
    end

    if TradeFill.itemdb.profile.itemIdA == 0 and TradeFill.itemdb.profile.itemIdB == 0 or TradeFill.itemdb.profile.button == true then
        ClearTrade:Hide()
    else
        ClearTrade:Show()
    end
end

function TradeFill:SetOptionStack(option)

    local selectOption = {}

    if option == "A" then
        for i = 0, NUM_TRADING_SLOTS do
            local itemId = TradeFill.itemdb.profile.itemIdA

            if itemId == 0 then
                selectOption[tostring(i)] = i
            else
                selectOption[tostring(i)] = i
            end
        end
    end

    if option == "B" then
        for i = 0, NUM_TRADING_SLOTS do
            local itemId = TradeFill.itemdb.profile.itemIdB

            if itemId == 0 then
                selectOption[tostring(i)] = i
            else
                selectOption[tostring(i)] = i
            end
        end
    end

    return selectOption
end

function TradeFill:SetOptionItem(option)

    for bag = 0, 4 do
        for slot = 1, C_Container.GetContainerNumSlots(bag) do
            local bagItem = C_Container.GetContainerItemInfo(bag, slot)

            if bagItem ~= nil then
                interfaceIndex = interfaceIndex + 1
                interfaceItem[interfaceIndex] = bagItem
            end
        end
    end

    local selectOption = {}

    selectOption["0"] = "None"

    for i, slot in pairs(interfaceItem) do
        local itemName, itemLink, _, _, _, _, _, itemStackCount, _, iconTexture = GetItemInfo(slot.itemID)

        if TradeFill.itemdb.profile.show == true then
            if slot.isBound == false and slot.stackCount == itemStackCount then
                selectOption[tostring(slot.itemID)] = "|T" .. GetItemIcon(slot.itemID) .. ":0|t " .. slot.hyperlink:gsub("[%[%]]", "")
            end
        else
            if slot.isBound == false then
                selectOption[tostring(slot.itemID)] = "|T" .. GetItemIcon(slot.itemID) .. ":0|t " .. slot.hyperlink:gsub("[%[%]]", "")
            end
        end

        if option == "A" and TradeFill.itemdb.profile.itemIdA > 0 then
            local item = TradeFill.itemdb.profile.itemIdA
            local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo(item)

            selectOption[tostring(item)] = "|T" .. GetItemIcon(item) .. ":0|t " .. itemLink:gsub("[%[%]]", "")
        end

        if option == "B" and TradeFill.itemdb.profile.itemIdB > 0 then
            local item = TradeFill.itemdb.profile.itemIdB
            local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo(item)

            selectOption[tostring(item)] = "|T" .. GetItemIcon(item) .. ":0|t " .. itemLink:gsub("[%[%]]", "")
        end
    end
    interfaceItem = {}
    interfaceIndex = 0
    return selectOption
end

function TradeFill:UpdateOptionStack(class, option, value)

    local stackA = self.stacksdb.profile[class].stackNumberA
    local stackB = self.stacksdb.profile[class].stackNumberB

    local newValue = NUM_TRADING_SLOTS - value

    if option == "A" then
        if stackA + stackB > NUM_TRADING_SLOTS then

            self.stacksdb.profile[class].stackNumberB = newValue
        end
    end

    if option == "B" then
        if stackA + stackB > NUM_TRADING_SLOTS then

            self.stacksdb.profile[class].stackNumberA = newValue
        end
    end
end

function TradeFill:GetInputItem(option)

    if option == "A" then
        local value = TradeFill.itemdb.profile.itemIdA
        TradeFill:enableDisableClassSetting(value, option)
        TradeFill:enableDisableButton(value, option)
        return tostring(value)
    end

    if option == "B" then
        local value = TradeFill.itemdb.profile.itemIdB
        TradeFill:enableDisableClassSetting(value, option)
        TradeFill:enableDisableButton(value, option)
        return tostring(value)
    end
end

function TradeFill:SetSelectItem(option, value)

    local item = itemData(value)

    if option == "A" then

        ButtonA:SetSize(32, 32)

        if value == 0 then
            ButtonA:SetNormalTexture(GetItemIcon(134400))
        else
            ButtonA:SetNormalTexture(GetItemIcon(value))
        end

        self.itemdb.profile.itemNameA = item.name
        self.itemdb.profile.itemIdA = value

        TradeFill:enableDisableClassSetting(value, option)
        TradeFill:enableDisableButton(value, option)
    end

    if option == "B" then

        ButtonB:SetSize(32, 32)

        if value == 0 then
            ButtonB:SetNormalTexture(GetItemIcon(134400))
        else
            ButtonB:SetNormalTexture(GetItemIcon(value))
        end

        self.itemdb.profile.itemNameB = item.name
        self.itemdb.profile.itemIdB = value

        TradeFill:enableDisableClassSetting(value, option)
        TradeFill:enableDisableButton(value, option)
    end
end

function TradeFill:GetName(option)

    local level = TradeFill.itemdb.profile.level

    if option == "A" then
        local item = itemData(self.itemdb.profile.itemIdA)

        if item.name then
            if level == true then
                return item.link:gsub("[%[%]]", "") .. " - Requires Level " .. item.level
            else
                return item.link:gsub("[%[%]]", "")
            end
        else
            return "Unknown"
        end
    end

    if option == "B" then
        local item = itemData(self.itemdb.profile.itemIdB)

        if item.name then
            if level == true then
                return item.link:gsub("[%[%]]", "") .. " - Requires Level " .. item.level
            else
                return item.link:gsub("[%[%]]", "")
            end
        else
            return "Unknown"
        end
    end
end

function TradeFill:SetIconImage(option)

    if option == "A" then
        local item = itemData(self.itemdb.profile.itemIdA)

        if item.name then
            return item.icon
        else
            return 134400
        end
    end

    if option == "B" then
        local item = itemData(self.itemdb.profile.itemIdB)

        if item.name then
            return item.icon
        else
            return 134400
        end
    end
end

LibStub("AceConfig-3.0"):RegisterOptionsTable("TradeFill", options)
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
AceConfigDialog:AddToBlizOptions("TradeFill", "TradeFill")

local defaultsStacks = {
    profile = {
        ["**"] = { stackNumberA = 0, stackNumberB = 0 }
    }
}

local defaultsItems = {
    profile = {
        itemIdA = 0, itemIdB = 0, show = false, stack = false, button = false, solo = false, party = false, raid = false, guild = false, players = "", guilds = "", level = false
    }
}

function TradeFill:OnInitialize()
    self.stacksdb = LibStub("AceDB-3.0"):New("StacksDB", defaultsStacks)
    self.itemdb = LibStub("AceDB-3.0"):New("ItemDB", defaultsItems)

    local faction = UnitFactionGroup("player")

    if faction == "Alliance" then
        options.args.stacks.args["SHAMAN"].hidden = true
    elseif faction == "Horde" then
        options.args.stacks.args["PALADIN"].hidden = true
    end

    OptionFrame:SetWidth(200)
    OptionFrame:SetHeight(600)
    OptionFrame:SetPoint("BOTTOMRIGHT", TradeFrame, "BOTTOMRIGHT", 0, 0)

    local itemIdA = TradeFill.itemdb.profile.itemIdA

    ButtonA:SetPoint("BOTTOM", OptionFrame, "BOTTOMRIGHT", 18, 62)

    if itemIdA == 0 then
        ButtonA:SetNormalTexture(GetItemIcon(134400))
    else
        ButtonA:SetNormalTexture(GetItemIcon(itemIdA))
    end

    ButtonA:SetSize(32, 32)
    local function ButtonItemA()
        tradeFill(itemData(TradeFill.itemdb.profile.itemIdA), tradePosition())
    end
    ButtonA:SetScript("OnClick", ButtonItemA)
    TradeFill:enableDisableButton(itemIdA, "A")

    local itemIdB = TradeFill.itemdb.profile.itemIdB

    ButtonB:SetPoint("BOTTOM", OptionFrame, "BOTTOMRIGHT", 18, 28)

    if itemIdB == 0 then
        ButtonB:SetNormalTexture(GetItemIcon(134400))
    else
        ButtonB:SetNormalTexture(GetItemIcon(itemIdB))
    end

    ButtonB:SetSize(32, 32)
    local function ButtonItemB()
        tradeFill(itemData(TradeFill.itemdb.profile.itemIdB), tradePosition())
    end
    ButtonB:SetScript("OnClick", ButtonItemB)
    TradeFill:enableDisableButton(itemIdB, "B")

    ClearTrade:SetText("Clear")
    ClearTrade:SetPoint("BOTTOMRIGHT", OptionFrame, "BOTTOMRIGHT", 73, 5)
    ClearTrade:SetSize(75, 22)
    local function ButtonClear()
        for i = 1, MAX_TRADABLE_ITEMS do
            ClearCursor()
            ClickTradeButton(i)
        end
        ClearCursor()
    end
    ClearTrade:SetScript("OnClick", ButtonClear)
end

ButtonA:SetScript("OnEnter", function(self)
    self:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
end)

ButtonB:SetScript("OnEnter", function(self)
    self:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
end)